/** ogftp.c
 **
 ** Omnigo/Unix file transfer program.
 **
 ** Written by and Copyright 1996 Asher Hoskins.
 ** Modified by Marcus Groeber for use under OS/2.
 **
 ** The author retains copyright on this implementation. Permission for
 ** educational and non-profit use is granted to all. If you're planning to
 ** make money with this or any code derived from it, check with the author
 ** first.
 **
 ** See README file for more information.
 **
 ** Main routine.
 **/


#include <stdio.h>
#ifdef OS2
	#include <io.h>
#endif
#include <ctype.h>
#include <string.h>
#include <fcntl.h>

#include "reply.h"
#include "transfer.h"
#include "open.h"

char *processCommand(int, char x[], char y[]);
void getCWD(int, char x[]);

char buf[65535];	/* Buffer to read results from Omnigo */


/** Main routine.
 **/

int main(int argc, char *argv[]) {

	int tty;
	unsigned int speed = DEFAULT_SPEED;
	char *port = DEFAULT_PORT;
	char inpline[1024];	/* Buffer for input line */
	char *inpres;

	puts("ogftp : Omnigo/Unix File Transfer Program");
	puts("        Written by Asher Hoskins, April 1996");
	puts("        Modified by Marcus Groeber for use under OS/2, May 1996\n");
	puts("Enter '?' for help.\n");

	if (argc>1)
		port = argv[1];
	if (argc>2)
		sscanf(argv[2], "%u", &speed);

	printf("Opening %s at %u bps...\n\n", port, speed);
	tty = openOmnigo(port, speed);

	write(tty, "\033EBON!", 6);
	readReply(tty, buf);
	write(tty, "\033CDB:\\geoworks\\document!", 24);
	readReply(tty, buf);
	do {
		printf("%s", buf);
		if ((inpres=gets(inpline)) != NULL)
			inpres=processCommand(tty, inpline, buf);
	} while (inpres != NULL);

	puts("\n\nDisconnecting...");
	write(tty, "\033EX", 3);
	read(tty, buf, 3);				/* Read <ESC>EX from Omnigo */
	close(tty);

	return(0);
}


/** Process a command entered by the user.
 ** Return NULL if command was 'QUIT' or EOF.
 **/

char *processCommand(int tty, char inpline[], char buf[]) {

	char *tok, *fname;
	unsigned int i, l;

	if (inpline[0] == '\0') {
		getCWD(tty,buf);
		return(inpline);
	}

	tok=strtok(inpline, " ");

	/* Convert command into upper case */
	l=strlen(tok);
	for (i=0; i<l; i++)
		if ((tok[i] >= 'a') && (tok[i] <= 'z'))
			tok[i] = toupper(tok[i]);

	if ((strcmp(tok, "?") == 0) || (strcmp(tok, "HELP") == 0)) {
		puts("\tls/dir              ... List current directory");
		puts("\tcd <dir>            ... Change directory to <dir>");
		puts("\tfree                ... Show free space on current drive");
		puts("\tdrives              ... List available drives");
		puts("\tdel <file>          ... Delete <file> from Omnigo");
		puts("\tren <from> <to>     ... Rename file");
		puts("\tcopy <from> <to>    ... Copy file");
		puts("\tmkdir <name>        ... Make directory <name> on Omnigo");
		puts("\trmdir <name>        ... Remove directory <name> from Omnigo");
		puts("\tget <file>          ... Get <file> from Omnigo");
		puts("\tput <file> [<rmt>]  ... Put <file> onto Omnigo (<rmt> = remote name)");
		#ifdef OS2
			puts("\tquit/<CTRL-Z>       ... Disconnect");
		#else
			puts("\tquit/<CTRL-D>       ... Disconnect");
		#endif
		getCWD(tty, buf);
		return(inpline);
	}

	if ((strcmp(tok, "LS") == 0) || (strcmp(tok, "DIR") == 0)) {
		write(tty, "\033LM!", 4);
		readReply(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "CD") == 0) {
		tok=strtok(NULL, " ");
		write(tty, "\033CD", 3);
		write(tty, tok, strlen(tok));
		write(tty, "!", 1);
		readReply(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "FREE") == 0) {
		write(tty, "\033FS!", 4);
		readReply(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "DRIVES") == 0) {
		write(tty, "\033AD!", 4);
		readReply(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "DEL") == 0) {
		tok=strtok(NULL, " ");
		write(tty, "\033RF", 3);
		write(tty, tok, strlen(tok));
		write(tty, "!", 1);
		readReply(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "REN") == 0) {
		tok=strtok(NULL, " ");
		write(tty, "\033MV", 3);
		write(tty, tok, strlen(tok));
		write(tty, "!", 1);
		tok=strtok(NULL, " ");
		write(tty, tok, strlen(tok));
		write(tty, "!", 1);
		readReply(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "COPY") == 0) {
		tok=strtok(NULL, " ");
		write(tty, "\033CP", 3);
		write(tty, tok, strlen(tok));
		write(tty, "!", 1);
		tok=strtok(NULL, " ");
		write(tty, tok, strlen(tok));
		write(tty, "!", 1);
		readReply(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "MKDIR") == 0) {
		tok=strtok(NULL, " ");
		write(tty, "\033MD", 3);
		write(tty, tok, strlen(tok));
		write(tty, "!", 1);
		readReply(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "RMDIR") == 0) {
		tok=strtok(NULL, " ");
		write(tty, "\033RD", 3);
		write(tty, tok, strlen(tok));
		write(tty, "!", 1);
		readReply(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "GET") == 0) {
		tok=strtok(NULL, " ");
		getFile(tty, buf, tok);
		getCWD(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "PUT") == 0) {
		fname=strtok(NULL, " ");
		tok=strtok(NULL, " ");
		if (tok == NULL)			/* Local name == remote name */
			tok=fname;
		putFile(tty, buf, fname, tok);
		getCWD(tty, buf);
		return(inpline);
	}

	if (strcmp(tok, "QUIT") == 0)
		return(NULL);

	puts("Unrecognized command.");
	getCWD(tty, buf);
	return(inpline);
}


/** Get a reply from the Omnigo just consisting of the current working
 ** directory.
 **/

void getCWD(int tty, char buf[]) {

	write(tty, "\033CD!", 4);
	readReply(tty, buf);
}

